from enum import StrEnum


class Icons(StrEnum):
    ADD = "\uea60"
    PLUS = "\uea60"
    GIST_NEW = "\uea60"
    REPO_CREATE = "\uea60"
    LIGHTBULB = "\uea61"
    LIGHT_BULB = "\uea61"
    REPO = "\uea62"
    REPO_DELETE = "\uea62"
    GIST_FORK = "\uea63"
    REPO_FORKED = "\uea63"
    GIT_PULL_REQUEST = "\uea64"
    GIT_PULL_REQUEST_ABANDONED = "\uea64"
    RECORD_KEYS = "\uea65"
    KEYBOARD = "\uea65"
    TAG = "\uea66"
    GIT_PULL_REQUEST_LABEL = "\uea66"
    TAG_ADD = "\uea66"
    TAG_REMOVE = "\uea66"
    PERSON = "\uea67"
    PERSON_FOLLOW = "\uea67"
    PERSON_OUTLINE = "\uea67"
    PERSON_FILLED = "\uea67"
    GIT_BRANCH = "\uea68"
    GIT_BRANCH_CREATE = "\uea68"
    GIT_BRANCH_DELETE = "\uea68"
    SOURCE_CONTROL = "\uea68"
    MIRROR = "\uea69"
    MIRROR_PUBLIC = "\uea69"
    STAR = "\uea6a"
    STAR_ADD = "\uea6a"
    STAR_DELETE = "\uea6a"
    STAR_EMPTY = "\uea6a"
    COMMENT = "\uea6b"
    COMMENT_ADD = "\uea6b"
    ALERT = "\uea6c"
    WARNING = "\uea6c"
    SEARCH = "\uea6d"
    SEARCH_SAVE = "\uea6d"
    LOG_OUT = "\uea6e"
    SIGN_OUT = "\uea6e"
    LOG_IN = "\uea6f"
    SIGN_IN = "\uea6f"
    EYE = "\uea70"
    EYE_UNWATCH = "\uea70"
    EYE_WATCH = "\uea70"
    CIRCLE_FILLED = "\uea71"
    PRIMITIVE_DOT = "\uea71"
    CLOSE_DIRTY = "\uea71"
    DEBUG_BREAKPOINT = "\uea71"
    DEBUG_BREAKPOINT_DISABLED = "\uea71"
    DEBUG_HINT = "\uea71"
    TERMINAL_DECORATION_SUCCESS = "\uea71"
    PRIMITIVE_SQUARE = "\uea72"
    EDIT = "\uea73"
    PENCIL = "\uea73"
    INFO = "\uea74"
    ISSUE_OPENED = "\uea74"
    GIST_PRIVATE = "\uea75"
    GIT_FORK_PRIVATE = "\uea75"
    LOCK = "\uea75"
    MIRROR_PRIVATE = "\uea75"
    CLOSE = "\uea76"
    REMOVE_CLOSE = "\uea76"
    X = "\uea76"
    REPO_SYNC = "\uea77"
    SYNC = "\uea77"
    CLONE = "\uea78"
    DESKTOP_DOWNLOAD = "\uea78"
    BEAKER = "\uea79"
    MICROSCOPE = "\uea79"
    VM = "\uea7a"
    DEVICE_DESKTOP = "\uea7a"
    FILE = "\uea7b"
    FILE_TEXT = "\uea7b"
    MORE = "\uea7c"
    ELLIPSIS = "\uea7c"
    KEBAB_HORIZONTAL = "\uea7c"
    MAIL_REPLY = "\uea7d"
    REPLY = "\uea7d"
    ORGANIZATION = "\uea7e"
    ORGANIZATION_FILLED = "\uea7e"
    ORGANIZATION_OUTLINE = "\uea7e"
    NEW_FILE = "\uea7f"
    FILE_ADD = "\uea7f"
    NEW_FOLDER = "\uea80"
    FILE_DIRECTORY_CREATE = "\uea80"
    TRASH = "\uea81"
    TRASHCAN = "\uea81"
    HISTORY = "\uea82"
    CLOCK = "\uea82"
    FOLDER = "\uea83"
    FILE_DIRECTORY = "\uea83"
    SYMBOL_FOLDER = "\uea83"
    LOGO_GITHUB = "\uea84"
    MARK_GITHUB = "\uea84"
    GITHUB = "\uea84"
    TERMINAL = "\uea85"
    CONSOLE = "\uea85"
    REPL = "\uea85"
    ZAP = "\uea86"
    SYMBOL_EVENT = "\uea86"
    ERROR = "\uea87"
    STOP = "\uea87"
    VARIABLE = "\uea88"
    SYMBOL_VARIABLE = "\uea88"
    ARRAY = "\uea8a"
    SYMBOL_ARRAY = "\uea8a"
    SYMBOL_MODULE = "\uea8b"
    SYMBOL_PACKAGE = "\uea8b"
    SYMBOL_NAMESPACE = "\uea8b"
    SYMBOL_OBJECT = "\uea8b"
    SYMBOL_METHOD = "\uea8c"
    SYMBOL_FUNCTION = "\uea8c"
    SYMBOL_CONSTRUCTOR = "\uea8c"
    SYMBOL_BOOLEAN = "\uea8f"
    SYMBOL_NULL = "\uea8f"
    SYMBOL_NUMERIC = "\uea90"
    SYMBOL_NUMBER = "\uea90"
    SYMBOL_STRUCTURE = "\uea91"
    SYMBOL_STRUCT = "\uea91"
    SYMBOL_PARAMETER = "\uea92"
    SYMBOL_TYPE_PARAMETER = "\uea92"
    SYMBOL_KEY = "\uea93"
    SYMBOL_TEXT = "\uea93"
    SYMBOL_REFERENCE = "\uea94"
    GO_TO_FILE = "\uea94"
    SYMBOL_ENUM = "\uea95"
    SYMBOL_VALUE = "\uea95"
    SYMBOL_RULER = "\uea96"
    SYMBOL_UNIT = "\uea96"
    ACTIVATE_BREAKPOINTS = "\uea97"
    ARCHIVE = "\uea98"
    ARROW_BOTH = "\uea99"
    ARROW_DOWN = "\uea9a"
    ARROW_LEFT = "\uea9b"
    ARROW_RIGHT = "\uea9c"
    ARROW_SMALL_DOWN = "\uea9d"
    ARROW_SMALL_LEFT = "\uea9e"
    ARROW_SMALL_RIGHT = "\uea9f"
    ARROW_SMALL_UP = "\ueaa0"
    ARROW_UP = "\ueaa1"
    BELL = "\ueaa2"
    BOLD = "\ueaa3"
    BOOK = "\ueaa4"
    BOOKMARK = "\ueaa5"
    DEBUG_BREAKPOINT_CONDITIONAL_UNVERIFIED = "\ueaa6"
    DEBUG_BREAKPOINT_CONDITIONAL = "\ueaa7"
    DEBUG_BREAKPOINT_CONDITIONAL_DISABLED = "\ueaa7"
    DEBUG_BREAKPOINT_DATA_UNVERIFIED = "\ueaa8"
    DEBUG_BREAKPOINT_DATA = "\ueaa9"
    DEBUG_BREAKPOINT_DATA_DISABLED = "\ueaa9"
    DEBUG_BREAKPOINT_LOG_UNVERIFIED = "\ueaaa"
    DEBUG_BREAKPOINT_LOG = "\ueaab"
    DEBUG_BREAKPOINT_LOG_DISABLED = "\ueaab"
    BRIEFCASE = "\ueaac"
    BROADCAST = "\ueaad"
    BROWSER = "\ueaae"
    BUG = "\ueaaf"
    CALENDAR = "\ueab0"
    CASE_SENSITIVE = "\ueab1"
    CHECK = "\ueab2"
    CHECKLIST = "\ueab3"
    CHEVRON_DOWN = "\ueab4"
    CHEVRON_LEFT = "\ueab5"
    CHEVRON_RIGHT = "\ueab6"
    CHEVRON_UP = "\ueab7"
    CHROME_CLOSE = "\ueab8"
    CHROME_MAXIMIZE = "\ueab9"
    CHROME_MINIMIZE = "\ueaba"
    CHROME_RESTORE = "\ueabb"
    CIRCLE_OUTLINE = "\ueabc"
    CIRCLE = "\ueabc"
    DEBUG_BREAKPOINT_UNVERIFIED = "\ueabc"
    TERMINAL_DECORATION_INCOMPLETE = "\ueabc"
    CIRCLE_SLASH = "\ueabd"
    CIRCUIT_BOARD = "\ueabe"
    CLEAR_ALL = "\ueabf"
    CLIPPY = "\ueac0"
    CLOSE_ALL = "\ueac1"
    CLOUD_DOWNLOAD = "\ueac2"
    CLOUD_UPLOAD = "\ueac3"
    CODE = "\ueac4"
    COLLAPSE_ALL = "\ueac5"
    COLOR_MODE = "\ueac6"
    COMMENT_DISCUSSION = "\ueac7"
    CREDIT_CARD = "\ueac9"
    DASH = "\ueacc"
    DASHBOARD = "\ueacd"
    DATABASE = "\ueace"
    DEBUG_CONTINUE = "\ueacf"
    DEBUG_DISCONNECT = "\uead0"
    DEBUG_PAUSE = "\uead1"
    DEBUG_RESTART = "\uead2"
    DEBUG_START = "\uead3"
    DEBUG_STEP_INTO = "\uead4"
    DEBUG_STEP_OUT = "\uead5"
    DEBUG_STEP_OVER = "\uead6"
    DEBUG_STOP = "\uead7"
    DEBUG = "\uead8"
    DEVICE_CAMERA_VIDEO = "\uead9"
    DEVICE_CAMERA = "\ueada"
    DEVICE_MOBILE = "\ueadb"
    DIFF_ADDED = "\ueadc"
    DIFF_IGNORED = "\ueadd"
    DIFF_MODIFIED = "\ueade"
    DIFF_REMOVED = "\ueadf"
    DIFF_RENAMED = "\ueae0"
    DIFF = "\ueae1"
    DIFF_SIDEBYSIDE = "\ueae1"
    DISCARD = "\ueae2"
    EDITOR_LAYOUT = "\ueae3"
    EMPTY_WINDOW = "\ueae4"
    EXCLUDE = "\ueae5"
    EXTENSIONS = "\ueae6"
    EYE_CLOSED = "\ueae7"
    FILE_BINARY = "\ueae8"
    FILE_CODE = "\ueae9"
    FILE_MEDIA = "\ueaea"
    FILE_PDF = "\ueaeb"
    FILE_SUBMODULE = "\ueaec"
    FILE_SYMLINK_DIRECTORY = "\ueaed"
    FILE_SYMLINK_FILE = "\ueaee"
    FILE_ZIP = "\ueaef"
    FILES = "\ueaf0"
    FILTER = "\ueaf1"
    FLAME = "\ueaf2"
    FOLD_DOWN = "\ueaf3"
    FOLD_UP = "\ueaf4"
    FOLD = "\ueaf5"
    FOLDER_ACTIVE = "\ueaf6"
    FOLDER_OPENED = "\ueaf7"
    GEAR = "\ueaf8"
    GIFT = "\ueaf9"
    GIST_SECRET = "\ueafa"
    GIST = "\ueafb"
    GIT_COMMIT = "\ueafc"
    GIT_COMPARE = "\ueafd"
    COMPARE_CHANGES = "\ueafd"
    GIT_MERGE = "\ueafe"
    GITHUB_ACTION = "\ueaff"
    GITHUB_ALT = "\ueb00"
    GLOBE = "\ueb01"
    GRABBER = "\ueb02"
    GRAPH = "\ueb03"
    GRIPPER = "\ueb04"
    HEART = "\ueb05"
    HOME = "\ueb06"
    HORIZONTAL_RULE = "\ueb07"
    HUBOT = "\ueb08"
    INBOX = "\ueb09"
    ISSUE_REOPENED = "\ueb0b"
    ISSUES = "\ueb0c"
    ITALIC = "\ueb0d"
    JERSEY = "\ueb0e"
    JSON = "\ueb0f"
    KEBAB_VERTICAL = "\ueb10"
    KEY = "\ueb11"
    LAW = "\ueb12"
    LIGHTBULB_AUTOFIX = "\ueb13"
    LINK_EXTERNAL = "\ueb14"
    LINK = "\ueb15"
    LIST_ORDERED = "\ueb16"
    LIST_UNORDERED = "\ueb17"
    LIVE_SHARE = "\ueb18"
    LOADING = "\ueb19"
    LOCATION = "\ueb1a"
    MAIL_READ = "\ueb1b"
    MAIL = "\ueb1c"
    MARKDOWN = "\ueb1d"
    MEGAPHONE = "\ueb1e"
    MENTION = "\ueb1f"
    MILESTONE = "\ueb20"
    GIT_PULL_REQUEST_MILESTONE = "\ueb20"
    MORTAR_BOARD = "\ueb21"
    MOVE = "\ueb22"
    MULTIPLE_WINDOWS = "\ueb23"
    MUTE = "\ueb24"
    NO_NEWLINE = "\ueb25"
    NOTE = "\ueb26"
    OCTOFACE = "\ueb27"
    OPEN_PREVIEW = "\ueb28"
    PACKAGE = "\ueb29"
    PAINTCAN = "\ueb2a"
    PIN = "\ueb2b"
    PLAY = "\ueb2c"
    RUN = "\ueb2c"
    PLUG = "\ueb2d"
    PRESERVE_CASE = "\ueb2e"
    PREVIEW = "\ueb2f"
    PROJECT = "\ueb30"
    PULSE = "\ueb31"
    QUESTION = "\ueb32"
    QUOTE = "\ueb33"
    RADIO_TOWER = "\ueb34"
    REACTIONS = "\ueb35"
    REFERENCES = "\ueb36"
    REFRESH = "\ueb37"
    REGEX = "\ueb38"
    REMOTE_EXPLORER = "\ueb39"
    REMOTE = "\ueb3a"
    REMOVE = "\ueb3b"
    REPLACE_ALL = "\ueb3c"
    REPLACE = "\ueb3d"
    REPO_CLONE = "\ueb3e"
    REPO_FORCE_PUSH = "\ueb3f"
    REPO_PULL = "\ueb40"
    REPO_PUSH = "\ueb41"
    REPORT = "\ueb42"
    REQUEST_CHANGES = "\ueb43"
    ROCKET = "\ueb44"
    ROOT_FOLDER_OPENED = "\ueb45"
    ROOT_FOLDER = "\ueb46"
    RSS = "\ueb47"
    RUBY = "\ueb48"
    SAVE_ALL = "\ueb49"
    SAVE_AS = "\ueb4a"
    SAVE = "\ueb4b"
    SCREEN_FULL = "\ueb4c"
    SCREEN_NORMAL = "\ueb4d"
    SEARCH_STOP = "\ueb4e"
    SERVER = "\ueb50"
    SETTINGS_GEAR = "\ueb51"
    SETTINGS = "\ueb52"
    SHIELD = "\ueb53"
    SMILEY = "\ueb54"
    SORT_PRECEDENCE = "\ueb55"
    SPLIT_HORIZONTAL = "\ueb56"
    SPLIT_VERTICAL = "\ueb57"
    SQUIRREL = "\ueb58"
    STAR_FULL = "\ueb59"
    STAR_HALF = "\ueb5a"
    SYMBOL_CLASS = "\ueb5b"
    SYMBOL_COLOR = "\ueb5c"
    SYMBOL_CONSTANT = "\ueb5d"
    SYMBOL_ENUM_MEMBER = "\ueb5e"
    SYMBOL_FIELD = "\ueb5f"
    SYMBOL_FILE = "\ueb60"
    SYMBOL_INTERFACE = "\ueb61"
    SYMBOL_KEYWORD = "\ueb62"
    SYMBOL_MISC = "\ueb63"
    SYMBOL_OPERATOR = "\ueb64"
    SYMBOL_PROPERTY = "\ueb65"
    WRENCH = "\ueb65"
    WRENCH_SUBACTION = "\ueb65"
    SYMBOL_SNIPPET = "\ueb66"
    TASKLIST = "\ueb67"
    TELESCOPE = "\ueb68"
    TEXT_SIZE = "\ueb69"
    THREE_BARS = "\ueb6a"
    THUMBSDOWN = "\ueb6b"
    THUMBSUP = "\ueb6c"
    TOOLS = "\ueb6d"
    TRIANGLE_DOWN = "\ueb6e"
    TRIANGLE_LEFT = "\ueb6f"
    TRIANGLE_RIGHT = "\ueb70"
    TRIANGLE_UP = "\ueb71"
    TWITTER = "\ueb72"
    UNFOLD = "\ueb73"
    UNLOCK = "\ueb74"
    UNMUTE = "\ueb75"
    UNVERIFIED = "\ueb76"
    VERIFIED = "\ueb77"
    VERSIONS = "\ueb78"
    VM_ACTIVE = "\ueb79"
    VM_OUTLINE = "\ueb7a"
    VM_RUNNING = "\ueb7b"
    WATCH = "\ueb7c"
    WHITESPACE = "\ueb7d"
    WHOLE_WORD = "\ueb7e"
    WINDOW = "\ueb7f"
    WORD_WRAP = "\ueb80"
    ZOOM_IN = "\ueb81"
    ZOOM_OUT = "\ueb82"
    LIST_FILTER = "\ueb83"
    LIST_FLAT = "\ueb84"
    LIST_SELECTION = "\ueb85"
    SELECTION = "\ueb85"
    LIST_TREE = "\ueb86"
    DEBUG_BREAKPOINT_FUNCTION_UNVERIFIED = "\ueb87"
    DEBUG_BREAKPOINT_FUNCTION = "\ueb88"
    DEBUG_BREAKPOINT_FUNCTION_DISABLED = "\ueb88"
    DEBUG_STACKFRAME_ACTIVE = "\ueb89"
    CIRCLE_SMALL_FILLED = "\ueb8a"
    DEBUG_STACKFRAME_DOT = "\ueb8a"
    TERMINAL_DECORATION_MARK = "\ueb8a"
    DEBUG_STACKFRAME = "\ueb8b"
    DEBUG_STACKFRAME_FOCUSED = "\ueb8b"
    DEBUG_BREAKPOINT_UNSUPPORTED = "\ueb8c"
    SYMBOL_STRING = "\ueb8d"
    DEBUG_REVERSE_CONTINUE = "\ueb8e"
    DEBUG_STEP_BACK = "\ueb8f"
    DEBUG_RESTART_FRAME = "\ueb90"
    DEBUG_ALT = "\ueb91"
    CALL_INCOMING = "\ueb92"
    CALL_OUTGOING = "\ueb93"
    MENU = "\ueb94"
    EXPAND_ALL = "\ueb95"
    FEEDBACK = "\ueb96"
    GIT_PULL_REQUEST_REVIEWER = "\ueb96"
    GROUP_BY_REF_TYPE = "\ueb97"
    UNGROUP_BY_REF_TYPE = "\ueb98"
    ACCOUNT = "\ueb99"
    GIT_PULL_REQUEST_ASSIGNEE = "\ueb99"
    BELL_DOT = "\ueb9a"
    DEBUG_CONSOLE = "\ueb9b"
    LIBRARY = "\ueb9c"
    OUTPUT = "\ueb9d"
    RUN_ALL = "\ueb9e"
    SYNC_IGNORED = "\ueb9f"
    PINNED = "\ueba0"
    GITHUB_INVERTED = "\ueba1"
    SERVER_PROCESS = "\ueba2"
    SERVER_ENVIRONMENT = "\ueba3"
    PASS = "\ueba4"
    ISSUE_CLOSED = "\ueba4"
    STOP_CIRCLE = "\ueba5"
    PLAY_CIRCLE = "\ueba6"
    RECORD = "\ueba7"
    DEBUG_ALT_SMALL = "\ueba8"
    VM_CONNECT = "\ueba9"
    CLOUD = "\uebaa"
    MERGE = "\uebab"
    EXPORT = "\uebac"
    GRAPH_LEFT = "\uebad"
    MAGNET = "\uebae"
    NOTEBOOK = "\uebaf"
    REDO = "\uebb0"
    CHECK_ALL = "\uebb1"
    PINNED_DIRTY = "\uebb2"
    PASS_FILLED = "\uebb3"
    CIRCLE_LARGE_FILLED = "\uebb4"
    CIRCLE_LARGE = "\uebb5"
    CIRCLE_LARGE_OUTLINE = "\uebb5"
    COMBINE = "\uebb6"
    GATHER = "\uebb6"
    TABLE = "\uebb7"
    VARIABLE_GROUP = "\uebb8"
    TYPE_HIERARCHY = "\uebb9"
    TYPE_HIERARCHY_SUB = "\uebba"
    TYPE_HIERARCHY_SUPER = "\uebbb"
    GIT_PULL_REQUEST_CREATE = "\uebbc"
    RUN_ABOVE = "\uebbd"
    RUN_BELOW = "\uebbe"
    NOTEBOOK_TEMPLATE = "\uebbf"
    DEBUG_RERUN = "\uebc0"
    WORKSPACE_TRUSTED = "\uebc1"
    WORKSPACE_UNTRUSTED = "\uebc2"
    WORKSPACE_UNKNOWN = "\uebc3"
    TERMINAL_CMD = "\uebc4"
    TERMINAL_DEBIAN = "\uebc5"
    TERMINAL_LINUX = "\uebc6"
    TERMINAL_POWERSHELL = "\uebc7"
    TERMINAL_TMUX = "\uebc8"
    TERMINAL_UBUNTU = "\uebc9"
    TERMINAL_BASH = "\uebca"
    ARROW_SWAP = "\uebcb"
    COPY = "\uebcc"
    PERSON_ADD = "\uebcd"
    FILTER_FILLED = "\uebce"
    WAND = "\uebcf"
    DEBUG_LINE_BY_LINE = "\uebd0"
    INSPECT = "\uebd1"
    LAYERS = "\uebd2"
    LAYERS_DOT = "\uebd3"
    LAYERS_ACTIVE = "\uebd4"
    COMPASS = "\uebd5"
    COMPASS_DOT = "\uebd6"
    COMPASS_ACTIVE = "\uebd7"
    AZURE = "\uebd8"
    ISSUE_DRAFT = "\uebd9"
    GIT_PULL_REQUEST_CLOSED = "\uebda"
    GIT_PULL_REQUEST_DRAFT = "\uebdb"
    DEBUG_ALL = "\uebdc"
    DEBUG_COVERAGE = "\uebdd"
    RUN_ERRORS = "\uebde"
    FOLDER_LIBRARY = "\uebdf"
    DEBUG_CONTINUE_SMALL = "\uebe0"
    BEAKER_STOP = "\uebe1"
    GRAPH_LINE = "\uebe2"
    GRAPH_SCATTER = "\uebe3"
    PIE_CHART = "\uebe4"
    BRACKET = "\ueb0f"
    BRACKET_DOT = "\uebe5"
    BRACKET_ERROR = "\uebe6"
    LOCK_SMALL = "\uebe7"
    AZURE_DEVOPS = "\uebe8"
    VERIFIED_FILLED = "\uebe9"
    NEWLINE = "\uebea"
    LAYOUT = "\uebeb"
    LAYOUT_ACTIVITYBAR_LEFT = "\uebec"
    LAYOUT_ACTIVITYBAR_RIGHT = "\uebed"
    LAYOUT_PANEL_LEFT = "\uebee"
    LAYOUT_PANEL_CENTER = "\uebef"
    LAYOUT_PANEL_JUSTIFY = "\uebf0"
    LAYOUT_PANEL_RIGHT = "\uebf1"
    LAYOUT_PANEL = "\uebf2"
    LAYOUT_SIDEBAR_LEFT = "\uebf3"
    LAYOUT_SIDEBAR_RIGHT = "\uebf4"
    LAYOUT_STATUSBAR = "\uebf5"
    LAYOUT_MENUBAR = "\uebf6"
    LAYOUT_CENTERED = "\uebf7"
    TARGET = "\uebf8"
    INDENT = "\uebf9"
    RECORD_SMALL = "\uebfa"
    ERROR_SMALL = "\uebfb"
    TERMINAL_DECORATION_ERROR = "\uebfb"
    ARROW_CIRCLE_DOWN = "\uebfc"
    ARROW_CIRCLE_LEFT = "\uebfd"
    ARROW_CIRCLE_RIGHT = "\uebfe"
    ARROW_CIRCLE_UP = "\uebff"
    LAYOUT_SIDEBAR_RIGHT_OFF = "\uec00"
    LAYOUT_PANEL_OFF = "\uec01"
    LAYOUT_SIDEBAR_LEFT_OFF = "\uec02"
    BLANK = "\uec03"
    HEART_FILLED = "\uec04"
    MAP = "\uec05"
    MAP_HORIZONTAL = "\uec05"
    FOLD_HORIZONTAL = "\uec05"
    MAP_FILLED = "\uec06"
    MAP_HORIZONTAL_FILLED = "\uec06"
    FOLD_HORIZONTAL_FILLED = "\uec06"
    CIRCLE_SMALL = "\uec07"
    BELL_SLASH = "\uec08"
    BELL_SLASH_DOT = "\uec09"
    COMMENT_UNRESOLVED = "\uec0a"
    GIT_PULL_REQUEST_GO_TO_CHANGES = "\uec0b"
    GIT_PULL_REQUEST_NEW_CHANGES = "\uec0c"
    SEARCH_FUZZY = "\uec0d"
    COMMENT_DRAFT = "\uec0e"
    SEND = "\uec0f"
    SPARKLE = "\uec10"
    INSERT = "\uec11"
    MIC = "\uec12"
    THUMBSDOWN_FILLED = "\uec13"
    THUMBSUP_FILLED = "\uec14"
    COFFEE = "\uec15"
    SNAKE = "\uec16"
    GAME = "\uec17"
    VR = "\uec18"
    CHIP = "\uec19"
    PIANO = "\uec1a"
    MUSIC = "\uec1b"
    MIC_FILLED = "\uec1c"
    REPO_FETCH = "\uec1d"
    COPILOT = "\uec1e"
    LIGHTBULB_SPARKLE = "\uec1f"
    ROBOT = "\uec20"
    SPARKLE_FILLED = "\uec21"
    DIFF_SINGLE = "\uec22"
    DIFF_MULTIPLE = "\uec23"
    SURROUND_WITH = "\uec24"
    SHARE = "\uec25"
    GIT_STASH = "\uec26"
    GIT_STASH_APPLY = "\uec27"
    GIT_STASH_POP = "\uec28"
    VSCODE = "\uec29"
    VSCODE_INSIDERS = "\uec2a"
    CODE_OSS = "\uec2b"
    RUN_COVERAGE = "\uec2c"
    RUN_ALL_COVERAGE = "\uec2d"
    COVERAGE = "\uec2e"
    GITHUB_PROJECT = "\uec2f"
    MAP_VERTICAL = "\uec30"
    FOLD_VERTICAL = "\uec30"
    MAP_VERTICAL_FILLED = "\uec31"
    FOLD_VERTICAL_FILLED = "\uec31"
    GO_TO_SEARCH = "\uec32"
    PERCENTAGE = "\uec33"
    SORT_PERCENTAGE = "\uec33"
    ATTACH = "\uec34"
