from biscuit.common.icons import Icons

kinds = (
    (Icons.SYMBOL_FILE, None),
    (Icons.SYMBOL_FILE, None),
    (Icons.SYMBOL_NAMESPACE, None),
    (Icons.PACKAGE, None),
    (Icons.SYMBOL_CLASS, "#de9327"),
    (Icons.SYMBOL_METHOD, "#AB7CCF"),
    (Icons.SYMBOL_PROPERTY, "#6eb1ee"),
    (Icons.SYMBOL_FIELD, "#6eb1ee"),
    (Icons.SYMBOL_METHOD, "#6eb1ee"),
    (Icons.SYMBOL_ENUM, None),
    (Icons.SYMBOL_INTERFACE, "#de9327"),
    (Icons.SYMBOL_METHOD, "#AB7CCF"),
    (Icons.SYMBOL_VARIABLE, "#6eb1ee"),
    (Icons.SYMBOL_CONSTANT, None),
    (Icons.SYMBOL_STRING, None),
    (Icons.SYMBOL_NUMERIC, None),
    (Icons.SYMBOL_BOOLEAN, "#de9327"),
    (Icons.SYMBOL_ARRAY, None),
    (Icons.SYMBOL_CLASS, None),
    (Icons.SYMBOL_KEY, None),
    (Icons.SYMBOL_MISC, None),
    (Icons.SYMBOL_ENUM_MEMBER, None),
    (Icons.SYMBOL_STRUCTURE, None),
    (Icons.SYMBOL_EVENT, "#ffd700"),
    (Icons.SYMBOL_OPERATOR, None),
    (Icons.SYMBOL_PARAMETER, "#ffd700"),
)
