from biscuit.common.icons import Icons

kinds = (
    (Icons.SYMBOL_KEY, None),  # text
    (Icons.SYMBOL_METHOD, "#AB7CCF"),  # method
    (Icons.SYMBOL_METHOD, "#AB7CCF"),  # function
    (Icons.SYMBOL_METHOD, "#6eb1ee"),  # constructor
    (Icons.SYMBOL_FIELD, "#6eb1ee"),  # field
    (Icons.SYMBOL_VARIABLE, "#6eb1ee"),  # variable
    (Icons.SYMBOL_CLASS, "#de9327"),  # class
    (Icons.SYMBOL_INTERFACE, "#6eb1ee"),  # interface
    (Icons.SYMBOL_NAMESPACE, None),  # module
    (Icons.SYMBOL_PROPERTY, "#6eb1ee"),  # property
    (Icons.SYMBOL_RULER, None),  # unit
    (Icons.SYMBOL_NUMERIC, None),  # value
    (Icons.SYMBOL_ENUM, None),  # enum
    (Icons.SYMBOL_KEYWORD, None),  # keyword
    (Icons.SYMBOL_SNIPPET, None),  # snippet
    (Icons.SYMBOL_COLOR, "#ffd700"),  # color
    (Icons.SYMBOL_FILE, None),  # file
    (Icons.SYMBOL_CLASS, None),  # reference
    (Icons.FOLDER, None),  # folder
    (Icons.SYMBOL_ENUM_MEMBER, None),  # enummember
    (Icons.SYMBOL_CONSTANT, None),  # constant
    (Icons.SYMBOL_STRUCTURE, None),  # struct
    (Icons.SYMBOL_EVENT, "#ffd700"),  # event
    (Icons.SYMBOL_OPERATOR, None),  # operator
    (Icons.SYMBOL_PARAMETER, "#ffd700"),  # typeparameter
)
